<?php
/**
 * Project: ePrint :: 확장 출력 API<br>
 * File:    ePrint.php
 *
 * 확장된 출력 stream API를 제공
 *
 * @category  Core
 * @package   ePrint
 * @author    JoungKyun.Kim <http://oops.org>
 * @copyright (c) 2014 JoungKyun.Kim
 * @license   BSD
 * @version   $Id: ePrint.php 41 2014-02-05 17:22:23Z oops $
 * @link      http://pear.oops.org/package/ePrint
 * @since     File available since relase 1.0.0
 * @example   pear_ePrint/test.php Sample codes for ePrint class
 * @filesource
 */

/**
 * 확장 출력 API를 위한 frontend 클래스
 * @package ePrint
 */
class ePrint {
	// {{{ properties
	const EPRINT_NORMAL  = 0;
	const EPRINT_WARNING = 1;
	const EPRINT_ERROR   = 2;

	/**
	 * 표준 입력 파일디스크립션을 할당할 변수 선언
	 *
	 * @var    resource
	 * @access public
	 */
	static public $stdin;
	/**
	 * 표준 출력 파일디스크립션을 할당할 변수 선언
	 *
	 * @var    resource
	 * @access public
	 */
	static public $stdout;
	/**
	 * 표준 에러 파일디스크립션을 할당할 변수 선언
	 *
	 * @var    resource
	 * @access public
	 */
	static public $stderr;

	/**
	 * ePrint::dprintf api에서 사용할 기본 디버그 레벨을 지정
	 *
	 * @var    int
	 * @access public
	 */
	static public $debugLevel = 0;
	static public $debugIP;
	// }}}

	// {{{ constructor
	/**
	 * @access public
	 * @return void
	 */
	function __construct () {
		self::regist ();
	}
	// }}}

	// {{{ (void) regist ($force = false)
	/**
	 * stdin/stdout/stderr 파일디스크립션을 등록
	 *
	 * 이 API는 cli 모드에서만 작동한다.
	 *
	 * @access public
	 * @return void
	 * @param boolean (optional) 본값 false. true로 설정되면 cli mode가
	 *                아니라도 작동
	 */
	static function regist ($force = false) {
		if ( php_sapi_name () == 'cli' )
			$force = true;

		if ( $force === false )
			return;

		self::$stdin = fopen ('php://stdin', 'rb');
		self::$stdout = fopen ('php://stdout', 'wb');
		self::$stderr = fopen ('php://stderr', 'wb');
	}
	// }}}

	// {{{ (void) whiteSpace ($no, $ret = false)
	/**
	 * 주어진 수 만큼의 공백 문자를 출력
	 *
	 * 예제:
	 * {@example pear_ePrint/test.php 22 8}
	 *
	 * @access public
	 * @return void    2번째 파라미터가 true로 지정이 되면, 문자열을 반환
	 * @param  integer 공백문자 수
	 * @param  boolean (optional) 기본값 <b>false</b>. true로 선언이 되면, 생성된
	 *                 공백문자를 출력하지 않고 반환함
	 */
	static function whiteSpace ($no, $ret = false) {
		if ( ! is_numeric ($no) )
			return '';

		if ( $ret === false ) {
			for ( $i=0; $i<$no; $i++ )
				echo ' ';
			return;
		}

		for ( $i=0; $i<$no; $i++ )
			$_r.= ' ';

		return $_r;
	}
	// }}}

	// {{{ (void) backSpace ($no, (resource) $handle = null)
	/**
	 * 주어진 수 만큼의 백스페이스를 출력
	 *
	 * 예제:
	 * {@example pear_ePrint/test.php 22 8}
	 *
	 * @access public
	 * @return void
	 * @param integer  출력할 bakspace 수
	 * @param resource (optional) fopen()으로 생성한 것과 같은 stream. 지정이
	 *                 되면, 이 stream으로 출력이 된다.
	 */
	static function backSpace ($no, $handle = null) {
		if ( ! is_numeric ($no) )
			return '';

		for ( $i=0; $i<$no; $i++ ) {
			if ( $handle === null )
				printf ('%c %c', '008', '008');
			else
				fprintf ($handle, '%c %c', '008', '008');
		}
		/*
		for ( $i=0; $i<$no; $i++ ) {
			echo "12" ^ "9";
			echo ' ';
			echo "12" ^ "9";
		}
		 */
	}
	// }}}

	// {{{ (int) ePrintf ($format, $msg = '', $type = 0, $des = '', $extra_header = '')
	/**
	 * 표준 에러로 형식화된 문자열을 출력
	 *
	 * {@link http://php.net/manual/en/function.error-log.php error_log()} 참조
	 *
	 * @access public
	 * @return integer 출력된 메세지의 길이
	 * @param  string  printf와 동일한 출력 형식
	 * @param  mixed   (optional) 형식의 인자 값<br>
	 *                 형식 아규먼트가 1개일 경우, $msg는 문자열 또는
	 *                 배열일 수 있다. 형식 아규먼트가 여러개일 경우,
	 *                 $msg 는 배열이어야 한다.
	 *
	 *                 예제:
	 *                 <code>
	 *                 ePrint::ePrintf ("%s %s %s", array ('a', 'b', 'c'));
	 *                 </code>
	 * @param  integer (optional) error_log의 2번째 인자 값과 동일. 기본값 <b>0</b><br>
	 *                 - <b>0:</b> apache sapi 에서는 error_log 로, cli 에서는
	 *                   stderr 로 출력
	 *                 - <b>1:</b> 4번째 인자($des)에 지정된 주소로 메일을 보냄. 5번째
	 *                   인자($extra_header)에 추가할 메일 헤더를 포함시킬 수
	 *                   있음
	 *                 - <b>2:</b> 사용하지 않음
	 *                 - <b>3:</b> 4번째 인자($des)에 지정된 파일에 내용을 추가함.
	 *                   메시지 끝에 newline이 붙이 않는 점을 주의
	 *                 - <b>4:</b> SAPI 로그 핸들러로 직접 보냄
	 * @param  string  (optional) error_log의 3번째 인자 값과 동일<br>
	 *                 3번째 인자($type)에 의존하며, 메일 주소나 저장할 파일을 지정
	 * @param  string  (optional) error_log의 4번째 인자 값과 동일<br>
	 *                 3번째 인자($type)의 값이 1일 경우 추가 메일 헤더를 지정할 수 있음
	 */
	static function ePrintf ($format, $msg = '', $type = 0, $des = '', $extra_header = '') {
		if ( $msg ) {
			$functionName = is_array ($msg) ? 'vsprintf' : 'sprintf';
			$format = $functionName ($format, $msg);
		}

		return error_log ($format, $type, $des, $extra_headers);
	}
	// }}}

	// {{{ (int) dPrintf ($leve, $format[, $var, ...])
	/**
	 * debug 용 printf method.
	 *
	 * ePrint::$debug_level이 첫 번째 인자값 보다 크거나 같을 경우
	 * 형식화된 문자열을 출력
	 *
	 * 예제:
	 * {@example pear_ePrint/test.php 31 5}
	 *
	 * @access public
	 * @return integer 출력한 문자열의 길이
	 * @param  integer $level  ePrint::$debug_level이 $level 보다 크거나
	 *                 같을 경우 출력
	 * @param  string  $format printf 의 문자열 형식과 동일
	 * @param  string  $args,...  (optional) 첫번째 문자열 형식의 값
	 */
	static function dPrintf () {
		$args= func_get_args ();
		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::dPrintf()');
			return;
		}

		$level = array_shift ($args);
		if ( self::$debugLevel < $level )
			return;

		return call_user_func_array (printf, $args);
	}
	// }}}

	// {{{ (int) lPrintf ($path, $format[, $var1, ...])
	/**
	 * 형식화된 문자열을 파일에 저장
	 * 
	 * format에 따른 출력을 파일에 저장 한다. 저장할 메시지 문자열 끝에
	 * newline이 자동으로 붙지 않는다.
	 *
	 * @access public
	 * @return integer 저장된 문자열의 길이
	 * @param  string  $path   저장할 파일경로
	 * @param  string  $format printf와 동일한 문자열 형식
	 * @param  string  $args,...   (optional) 형식 값
	 */
	static function lPrintf () {
		$args= func_get_args ();
		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::lPrintf()');
			return;
		}

		$f = array_shift ($args);
		$format = array_shift ($args);
		return self::ePrintf ($format, $args, 3, $f);
	}
	// }}}

	// {{{ (int) function fprintf ((resource) $handle, $format, $var, ...) {
	/*
	static function fprintf () {
		$args= func_get_args ();

		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::fprintf()');
			return;
		}

		if ( function_exists ('fprintf') )
			return call_user_func_array (fprintf, $args);

		$handle =  $array_shift ($args);
		if ( ! is_resource ($handle) ) {
			self::warning ('fprintf() supplied argument is not a valid stream resource');
			return;
		}
		$format = array_shift ($args);

		return fwrite ($handle, vsprintf ($format, $args));
	}
	 */
	// }}}

	// {{{ (int) printi ($msg, $width = 80, $indent = 0, $ul = '', $to_stderr = false)
	/**
	 * 주어진 수 만큼의 indent를 포함하여 출력
	 *
	 * 예제:
	 * {@example pear_ePrint/test.php 37 9}
	 *
	 * @access public
	 * @return integer|false 출력한 문자열의 길이를 반환 한다. 이 method가
	 *                       수행 중에 에러가 발생할 경우 false를 반환 한다.
	 * @param  mixed   출력할 문자열
	 * @param  integer (optional) 라인 넘김 위치. 기본값 <b>80</b>컬럼.<br>
	 *                 주어진 값 + 1 컬럼 위치에 newline이 추가된다.
	 * @param  integer (optional) 각 라인에 추가할 indent 수
	 * @param  string  (optional) list 표시 문자 (HTML의 li tag 처럼..)
	 * @param  boolean (optional) true로 설정이 되면 표준 에러로 출력.
	 *                 기본값 <b>false</b>
	 */
	static function printi ($msg, $width = 80, $indent = 0, $ul = '', $to_stderr = false) {
		if ( $width === 0 )
			$width = 10000; /* means unlimits */

		if ( $indent > 0 ) {
			$width -= $indent;
			if ( $width <= 0 ) {
				self::warning ('Plez make string lenght more than current length for ePrint::wordwrap()');
				return false;
			}
		}

		$l = strlen ($ul);
		if ( $l > 0 ) {
			$width -= $l + 1;
			$ul .= ' ';
		}

		if ( is_array ($msg) ) {
			if ( !count ($msg) )
				return;

			foreach ($msg as $v)
				$_msg .= rtrim ($v) . "\n";
		} else {
			if ( ! trim ($msg) )
				return;

			$_msg = preg_replace ('/[\r]/', '', $msg);
		}

		unset ($msg);
		$msg = rtrim (self::wordwrap ($_msg, $width));
		if ( preg_match ('/-newline$/', $msg) ) {
			$msg = preg_replace ('/-newline$/', '', $msg);
			$new_line = 0;
		} else {
			$new_line = 1;
		}

		unset ($_msg);
		$_msg = preg_split ("/[\n]/", $msg);
		$_line = count ($_msg);

		for ( $i=0; $i<$_line; $i++ ) {
			if ( $i == 1 ) {
				$_ul = strlen ($ul);
				$ul = str_repeat (' ', $_ul);
			}
			if ( $to_stderr ) {
				self::ePrintf ("%s%s%s", array (str_repeat (' ', $indent), $ul, $_msg[$i]));
			} else {
				printf ("%s%s%s%s",
						str_repeat (' ', $indent),
						$ul,
						str_replace ('&nbsp;', ' ', $_msg[$i]),
						$new_line ? "\n" : '');
			}
		}

		return true;
	}
	// }}}

	// {{{ (void) echoi {$msg, $indent = 0)
	/**
	 * 주어진 수만큼 indent를 추가하고 지정된 메시지를 출력
	 *
	 * ePrint::printi와의 차이는 line break가 자동으로 들어가지 않는다.
	 *
	 * @access public
	 * @return void
	 * @param  mixed 출력할 문자열
	 * @param  int   (optional) 각 라인의 indent 수. 기본값 <b>0</b>
	 */
	static function echoi ($msg, $indent = 0) {
		if ( ! is_array ($msg) ) {
			$msg = preg_replace ('/^/m', str_repeat (' ', $indent), $msg);
			echo $msg;
			return;
		}

		$no = count ($msg);
		$i = 1;
		foreach ( $msg as $v ) {
			if ( $i++ !== $no )
				$v = rtrim ($v) . "\n";
			printf ("%s%s", str_repeat (' ', $indent), $v);
		}
	}
	// }}}

	// {{{ (string) wordwrap ($msg, $width = 75, $break = "\n", $cut = 0)
	/**
	 * 주어진 수 만큼의 위치에 line break를 추가
	 *
	 * PHP의 wordwrap과의 차이는 line break가 추가 되었을 경우, line break
	 * 된 라인의 경우 다음 라인과 합쳐져서 다시 wrapping 을 한다.
	 *
	 * 만약 라인 채우기를 원하지 않은 라인이 있을 경우 라인 마지막에 '-*FN*-'
	 * 표시를 하면, 이 표시는 제거되고 라인 채우기를 하지 않는다.
	 *
	 * @access public
	 * @return strings
	 * @param  string  입력 문자열
	 * @param  integer (optional) 컬럼 위치. 지정된 컬럼 다음에 line break가
	 *                 온다.
	 * @param  string  (optional) 기본값 <b>'\n'</b>. 기본값을 변경할 경우
	 *                 원하는 string으로 지정할 수 있다.
	 * @param  integer (optional) TRUE로 지정이 되면 문자열은 지정된 위치 또는
	 *                 그 앞에서 무조건 강제 줄넘김을 추가 하게 됩니다. 주어진
	 *                 수 보다 긴 단어가 존재 할 경우, 단어 중간에서 줄넘김이
	 *                 행해 집니다. php의
	 *                 {@link http://php.net/manual/en/function.wordwrap.php wordwarp()}의
	 *                 4번째 인자에 대한 설명을 참조 하십시오.
	 */
	static function wordwrap ($msg, $width = 75, $break = "\n", $cut = 0) {
		$msg1 = iconv ('utf-8', 'cp949', $msg);
		$msg = wordwrap ($msg1 ? $msg1 : $msg, $width, $break, $cut);

		$break = preg_quote ($break);
		$_msg = preg_split ("/[{$break}]/", $msg);
		$_msgl = count ($_msg);

		for ( $i=0; $i<$_msgl; $i++ ) {
			$current = rtrim ($_msg[$i]);
			$l = strlen ($current);
			$chk = $width - $l;

			if ( $i == 0 ) {
				$v = $current . "\n";
				continue;
			}

			if ( preg_match ('/-\*FN\*-/', $current) ) {
				$current = preg_replace ('/-\*FN\*-/', '', $current);
				$chk = $width;
			}

			if ( $chk <= ($width * 0.1) || $chk == $width ) {
				$v .= $current . "\n";
			} else {
				if ( $i == ($_msgl - 1) ) {
					$v .= $current . "\n"; 
					break;
				}

				$next = rtrim ($_msg[++$i]);

				if ( ! strlen ($next) ) {
					$v .= $current . "\n\n";
					continue;
				}

				$current .= ' ' . $next;

				$current = wordwrap ($current, $width, $break, $cut);
				$_c = preg_split ("/[{$break}]/", $current);
				$_cl = count ($_c);

				$v .= $_c[0] . "\n";

				$_cl--;
				for ( $j=1; $j<=$_cl; $j++ ) {
					$_msg[$i - ($_cl - $j)] = $_c[$j];
					$i--;
				}
			}
		}

		return $msg1 ? iconv ('cp949', 'utf-8', $v) : $v;
	}
	// }}}

	// {{{ (void) warning ($msg, $type = self::EPRINT_WARNING)
	/**
	 * 경고 메시지 출력
	 *
	 * ePrint 또는 기타 다른 Pear 패키지에서 사용자 레벨의 php 경고/에러
	 * 메시지를 출력한다.
	 *
	 * @access public
	 * @return void
	 * @param  string  출력할 문자열. 문자열의 처음이 /^([^:]+)::/ 와 같이
	 *                 시작한다면, 정규식 $1에 매치되는 부분의 문자열이 에러
	 *                 메시지의 pear 이름 대신 사용한다.
	 * @param  integer (optional)
	 *                 - <b>ePrint::EPRINT_WARNING</b> : php 경고 메시지로 출력 / 기본값
	 *                 - ePrint::EPRINT_ERROR   : php 에러 메시지로 출력
	 */
	static function warning ($msg, $type = self::EPRINT_WARNING) {
		$t = ($type === self::EPRINT_WARNING) ? 'Warnging' : 'Error';

		$dinfo = debug_backtrace ();
		$file = $dinfo[1]['file'];
		$line = $dinfo[1]['line'];
		$postfix = sprintf (' in %s on line %d', $file, $line);

		if ( preg_match ('/^([^:\s]+)::(.+)/', $msg, $matches) ) {
			$pearName = $matches[1];
			$msg = $matches[2];
		} else
			$pearName = 'ePrint';

		$prefix = "PEAR::${pearName} {$t}: ";
		$format = $prefix . $msg . $postfix;

		self::ePrintf ($format);

		if ( $type === self::EPRINT_ERROR )
			exit (1);
	}
	// }}}

	// {{{ +-- static public (void) trim (&$v)
	/**
	 * Strip whitespace (or other characters) from the beginning and end of a string
	 * 이 method는 return 값이 없으며, 입력된 변수를 직접 수정한다.
	 *
	 * @access public
	 * @return void
	 * @param  string
	 * @since 1.0.5
	 */
	static public function trim (&$v) {
		$v = trim ($v);
	}
	// }}}

	// {{{ +-- static public (void) ltrim (&$v)
	/**
	 * Strip whitespace (or other characters) from the beginning of a string
	 * 이 method는 return 값이 없으며, 입력된 변수를 직접 수정한다.
	 *
	 * @access public
	 * @return void
	 * @param  string
	 * @since 1.0.5
	 */
	static public function ltrim (&$v) {
		$v = ltrim ($v);
	}
	// }}}

	// {{{ +-- static public (void) rtrim (&$v)
	/**
	 * Strip whitespace (or other characters) from the end of a string
	 * 이 method는 return 값이 없으며, 입력된 변수를 직접 수정한다.
	 *
	 * @access public
	 * @return void
	 * @param  string
	 * @since 1.0.5
	 */
	static public function rtrim (&$v) {
		$v = rtrim ($v);
	}
	// }}}

	/*
	 * Ansi print
	 */

	// {{{ (string) setColor ($color)
	/**
	 * 색상 문자열에 대한 안시 코드를 반환
	 *
	 * @access public
	 * @return string 주어진 색상에 대한 안시 코드.
	 * @param  string (optional) 안시 코드에 대한 색상 문자열.
	 *
	 *                다음의 색상 문자열을 사용할 수 있다:
	 *
	 *                - BLUE
	 *                - CYAN
	 *                - <b>GRAY</b> (기본값)
	 *                - GREEN
	 *                - MEGENTA
	 *                - RED
	 *                - WHITE
	 *                - YELLOW
	 *                - END
	 *
	 *                상값은 대소문자 구분을 하지 않는다.
	 */
	static function setColor ($color) {
		$color = $color ? strtolower ($color): 'gray';

		switch ( $color ) {
			case 'blue'    : return '[1;34m'; break;
			case 'cyan'    : return '[1;36m'; break;
			case 'gray'    : return '[1;30m'; break;
			case 'green'   : return '[1;32m'; break;
			case 'megenta' : return '[1;35m'; break;
			case 'red'     : return '[1;31m'; break;
			case 'white'   : return '[1;37m'; break;
			case 'yellow'  : return '[1;33m'; break;
			case 'end'     : return '[7;0m';  break;
		}

		/* defaults to gray code */
		return '[1;30m';
	}
	// }}}

	// {{{ (int) aPrintf ($color, $format[, $var, ...])
	/**
	 * 안시 코드를 수반한 형식화된 문자열을 출력
	 *
	 * 예제:
	 * {@example pear_ePrint/test.php 47 6}
	 *
	 * @access public
	 * @return integer 출력한 문자열의 길이
	 * @param  string  $color  출력할 안시 코드 색상 문자열. {@link ePrint::setColor()} 참조
	 * @param  string  $format printf의 format과 동일
	 * @param  string  $args,... (optional) 형식 값
	 */
	static function aPrintf () {
		$args= func_get_args ();

		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::aPrintf()');
			return;
		}

		$color = array_shift ($args);
		$start = self::setColor ($color);
		$end   = self::setColor	('end');

		$format = array_shift ($args);
		$format = '%s' . $format . '%s';
		$_args = array_merge (array ($start), $args, array ($end));

		return vprintf ($format, $_args);
	}
	// }}}

	// {{{ (string) asPrintf ($color, $format[, $var, ...])
	/**
	 * 안시 코드를 수반한 형식화된 문자열을 반환.
	 *
	 * 예제:
	 * {@example pear_ePrint/test.php 47 6}
	 *
	 * @access public
	 * @return string
	 * @param  string $color  안시 코드 색상 이름. {@link ePrint::setColor()} 참조
	 * @param  string $format sprintf의 format과 동일
	 * @param  string $args,...  (optional) 형식 값
	 */
	static function asPrintf () {
		$args= func_get_args ();

		if ( count ($args) < 2 ) {
			self::warning ('Wrong parameter count for ePrint::aPrintf()');
			return;
		}

		$color = array_shift ($args);
		$start = self::setColor ($color);
		$end   = self::setColor	('end');

		$format = array_shift ($args);
		$format = '%s' . $format . '%s';
		$_args = array_merge (array ($start), $args, array ($end));

		return vsprintf ($format, $_args);
	}
	// }}}

	// {{{ (void) unregist (void)
	/**
	 * stdin/stdout/stderr 파일 디스크립터를 해제
	 * 
	 * @access public
	 * @return void
	 */
	static function unregist () {
		if ( is_resource (self::$stdin) )
			fclose (self::$stdin);
		if ( is_resource (self::$stdout) )
			fclose (self::$stdout);
		if ( is_resource (self::$stderr) )
			fclose (self::$stderr);
	}
	// }}}

	// {{{ destructor
	function __destruct () {
		self::unregist();
	}
	// }}}
}

?>
